package scalewaysdkgo

// This test file makes sure that all the auto-generated code compiles.

import (
	_ "github.com/scaleway/scaleway-sdk-go/api/account/v3"
	_ "github.com/scaleway/scaleway-sdk-go/api/applesilicon/v1alpha1"
	_ "github.com/scaleway/scaleway-sdk-go/api/baremetal/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/billing/v2beta1"
	_ "github.com/scaleway/scaleway-sdk-go/api/block/v1alpha1"
	_ "github.com/scaleway/scaleway-sdk-go/api/cockpit/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/container/v1beta1"
	_ "github.com/scaleway/scaleway-sdk-go/api/dedibox/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/documentdb/v1beta1"
	_ "github.com/scaleway/scaleway-sdk-go/api/domain/v2beta1"
	_ "github.com/scaleway/scaleway-sdk-go/api/edge_services/v1alpha1"
	_ "github.com/scaleway/scaleway-sdk-go/api/flexibleip/v1alpha1"
	_ "github.com/scaleway/scaleway-sdk-go/api/function/v1beta1"
	_ "github.com/scaleway/scaleway-sdk-go/api/iam/v1alpha1"
	_ "github.com/scaleway/scaleway-sdk-go/api/inference/v1beta1"
	_ "github.com/scaleway/scaleway-sdk-go/api/instance/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/iot/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/ipam/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/ipam/v1alpha1"
	_ "github.com/scaleway/scaleway-sdk-go/api/k8s/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/key_manager/v1alpha1"
	_ "github.com/scaleway/scaleway-sdk-go/api/lb/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/marketplace/v2"
	_ "github.com/scaleway/scaleway-sdk-go/api/mnq/v1beta1"
	_ "github.com/scaleway/scaleway-sdk-go/api/mongodb/v1alpha1"
	_ "github.com/scaleway/scaleway-sdk-go/api/qaas/v1alpha1"
	_ "github.com/scaleway/scaleway-sdk-go/api/rdb/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/redis/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/registry/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/secret/v1beta1"
	_ "github.com/scaleway/scaleway-sdk-go/api/serverless_sqldb/v1alpha1"
	_ "github.com/scaleway/scaleway-sdk-go/api/tem/v1alpha1"
	_ "github.com/scaleway/scaleway-sdk-go/api/test/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/vpc/v2"
	_ "github.com/scaleway/scaleway-sdk-go/api/vpcgw/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/webhosting/v1"
	_ "github.com/scaleway/scaleway-sdk-go/api/webhosting/v1alpha1"
)
