// Package scalewaysdkgo is the Scaleway API SDK for Go.
//
// In order to use the available APIs, create a `Client`. Once created, it can be used to instantiate an API.
// To use the `instance` API, for example, instantiate it (with the client object) `instance.NewApi(client)`.
// On this instance API, all the available API functions can be called.
package scalewaysdkgo
